(*
copyright 2001 John Langford (jcl@cs.cmu.edu)
This code is available under GPL.
*)

let default_perm = 384

let make_hash_table name = 
  Dbm.opendbm name (Dbm.Dbm_create::Dbm.Dbm_rdwr::[]) default_perm 
    
let get database key = 
  try int_of_string (Dbm.find database key) 
  with Not_found -> 0 

let max = 1 lsl 22 - 1

let decay database time_scale = 
  let pairs = Array.create max ("",0) in
  let point = ref 0 in

  Dbm.iter (fun hash count -> 
    if !point < max then begin
      pairs.(!point) <- hash,int_of_string count;
      incr point; end
    else ()) database;
  
  let lost = ref 0 in
  

  for i = 0 to !point - 1 do
    let hash,count = pairs.(i) in
    let new_count = count * (time_scale-1) / time_scale
    and prob = (count * (time_scale-1)) mod time_scale in
    let new_count = new_count + 
      if Random.int time_scale < prob then 1
      else 0 in
    lost := !lost + (count - new_count);
    if new_count = 0 
    then Dbm.remove database hash
    else Dbm.replace database hash (string_of_int new_count) 
  done;
  print_int !lost;
  print_string " decayed away\n"
    
	
let get_incr database key = 
  let prev = get database key in
  Dbm.replace database key (string_of_int (prev+1));
  prev
    
let close database = Dbm.close database
